/**
 * @file raytime.c
 * This is the main raytime program start point.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <propeller.h>
#include <sys/rtc.h>

#define BUFFERLEN 10 // use for prompttime buffer size

void pause(int sec);
void printtime(void);
int  getdst(void);
int  prompttime(char *prompt);
void settime(void);

/**
 * Main program function.
 */
int main(void)
{
    putchar(0); // silence that noisy "QuickStart" board.
    _rtc_start_timekeeping_cog();
    pause(1);   // wait for terminal to start.
    printf("Propeller RTC SetTime Demo.\n");

    settime();
    for(;;) {
        pause(1);
        printtime();
    }
    return 0;
}

/**
 * Pause number of seconds based on Propeller system clock.
 * This function is not pthread safe.
 */
void pause(int secs)
{
    while (secs > 0) {
        waitcnt(CLKFREQ+CNT);
        --secs;
    }
}

/**
 * Print library's interpretation of time for human consumption.
 */
void printtime(void)
{
    time_t now;
    now = time(NULL);
    printf("%s", asctime(localtime(&now)));
}

/**
 * prompt for daylight savings time
 */
int getdst(void)
{
    int rc = 0;

    printf("Use daylight savings time [y/n] ? ");
    fflush(stdout);

    /* check for daylight savings time */
    rc = (getchar() == 'y') ? 1 : 0;

    /* swallow getchar() end of line */
    getchar();

    return rc;
}

/**
 * Prompt for time entry. Checks for invalid number.
 */
int prompttime(char *prompt)
{
    int  rc = 0;
    char *endp;
    char buffer[BUFFERLEN];

    do {
        printf("Enter %s: ",prompt);
        fflush(stdout);

        fgets(buffer,BUFFERLEN,stdin);
        fflush(stdin);

        /* Use strtol so we can error check input.
         * It will throw away any leading space or tab.
         * Parsing will stop at the first non-decimal numeric character.
         * If buffer == endp (checking address), no number is detected.
         */
        rc = strtol(buffer, &endp, 10);

        /* Did we get any valid number?
         */
        if(endp == buffer) {
            if('\n' == *endp) {
                rc = 0;
                break;
            }
            printf("Invalid entry \"%c....\" Please enter a number.\n", *endp);
        }
    } while (endp == buffer);
    
    return rc;
}

/**
 * set time for RTC
 */
void settime(void)
{
    struct timeval tv;
    struct tm t;

    /* get date/time info */
    t.tm_isdst = getdst();
    t.tm_year  = prompttime("Year")-1900;
    t.tm_mon   = prompttime("Month")-1;
    t.tm_mday  = prompttime("Day of the month");
    t.tm_hour  = prompttime("Hour");
    t.tm_min   = prompttime("Minute");
    t.tm_sec   = prompttime("Second");

    tv.tv_sec = mktime(&t);
    settimeofday(&tv, 0);

    printf("Set time all done.\n");
}